 
//  Copyright (c) 1994 by Mike Romberg (  romberg@md.fsl.noaa.gov )
//  This file may be distributed under terms of the GPL
//

#include "record.h"
#include "xtypewatch.h"
#include "global.h"
#include "speedmeter.h"
#include "breakmeter.h"
#include <iostream.h>
#include <unistd.h>
#include <string.h>

#include <stdlib.h>
#include <sys/time.h>



static const char NAME[] = "xtypewatch@";


XTypeWatch::XTypeWatch( int argc, char *argv[] ) : XWin( argc, argv, NULL ){
    cerr << "Xtypewatch is distributed under terms of the General Public License. \n"
	"It comes with NO WARRANTY. Use at your own risk"<<endl;





    xoff_ = 5;
    yoff_ = 0;
    nummeters_ = 0;
    meters_ = NULL;
    name_ = "xtypewatch";
    recorder_ = new Recorder(BUFFERLEN);
    myTime_ = 0;
    //  set up the X events
    addEvent( new Event( this, ConfigureNotify, 
			 (EventCallBack)&XTypeWatch::resizeEvent ) );
    addEvent( new Event( this, Expose, 
			 (EventCallBack)&XTypeWatch::exposeEvent ) );
    addEvent( new Event( this, KeyPress, 
			 (EventCallBack)&XTypeWatch::keyPressEvent ) );
 
    addEvent( new Event( this, CreateNotify,
			 (EventCallBack)&XTypeWatch::winCreateEvent ));

  
  
    // add or change the Resources
    addResource( new Parm( "labels", "-l", "False", 1 ) );
    addResource( new Parm( "usedlabels", "-ul", "False", 1 ) );

    addResource( new Parm( "microBreak", "-mb", "0,0,0", 0 ) );
    addResource( new Parm( "break", "-b", "0,0,0", 0) );
    addResource( new Parm( "ignore", "-ignore", "Xdvi,Ghostview", 0) );
   
    changeResource( "background", "navy" );
    changeResource( "foreground", "cyan" );
    changeResource( "font", "7x13bold" );
    select_recorderEvents();
  
    // see if legends are to be used
    parseargs( argc, argv );
    if ( legend_ ){
	if ( !usedlabels_ )
	    xoff_ = textWidth( "XXXXXXX" );
	else
	    xoff_ = textWidth( "XXXXXXXXXXX" );

	yoff_ = textHeight() + textHeight() / 4;
    }


  
  
    addmeter(new SpeedMeter(this, xoff_, newypos(), width_ - 10, 10));
    if (micro_.duration_ > 0){
	addmeter(new BreakMeter(this, micro_, xoff_, newypos(), width_ - 10, 10));
	recorder_->add_break(micro_);
    }

    if (normal_.duration_ > 0) {
	addmeter(new BreakMeter(this, normal_, xoff_, newypos(), width_ - 10, 10));
	recorder_->add_break(normal_);
    }
  
    // check for new resources that the meters may have added
    parseargs( argc, argv );
    //  dumpResources( cerr );
    checkMeterResources();
    //  dumpResources( cerr );

    // determine the width and height of the window then create it
    width_ = findx();
    height_ = findy();
    init( argc, argv );
    title( winname() );
    iconname( winname() );
    dolegends();
    resize();

}

void XTypeWatch::checkMeterResources( void )
{
    MeterNode *tmp = meters_;

    while ( tmp != NULL ){
	tmp->meter_->checkResources();
	tmp = tmp->next_;
    }
}
int XTypeWatch::newypos( void )
{
    return 15 + 25 * nummeters_;
}

void XTypeWatch::dolegends( void )
{
    
    MeterNode *tmp = meters_;
    while ( tmp != NULL ){
	tmp->meter_->dolegends( legend_ );
	tmp->meter_->dousedlegends( usedlabels_ );
	tmp = tmp->next_;
    }
}

void XTypeWatch::addmeter( Meter *fm ){
    MeterNode *tmp = meters_;

    if ( meters_ == NULL )
	meters_ = new MeterNode( fm );
    else {
	while ( tmp->next_ != NULL )
	    tmp = tmp->next_;
	tmp->next_ = new MeterNode( fm );
    }
    nummeters_++;
}

int XTypeWatch::findx( void ){
    if ( legend_ ){
	if ( !usedlabels_ )
	    return textWidth( "XXXXXXXXXXXXXXXXXXXXXXXX" );
	else
	    return textWidth( "XXXXXXXXXXXXXXXXXXXXXXXXXXXXX" );
    }
    return 80;
}

int XTypeWatch::findy( void ){
    if ( legend_ )
	return 10 + textHeight() * nummeters_ * 2;

    return 15 * nummeters_;
}

void
XTypeWatch::track_window(Window w)
{
    if (w == window_)		// don't want to mess with my own window
	return ;

    unsigned M = KeyPressMask;
    
    // report all newly created windows.
    if (w == DefaultRootWindow(display_))
	M |=SubstructureNotifyMask;

    XWindowAttributes    xAttr;
    XSetWindowAttributes xSwAttr;
    bool ok= XGetWindowAttributes( display_, w, &xAttr ) ;
    if (!ok)			//  can't get it's attributes, so try this.
	XSelectInput(display_, w,M);
    else {
	xSwAttr.event_mask = xAttr.your_event_mask | M;
	XChangeWindowAttributes( display_, w, CWEventMask, &xSwAttr );
    }    
}

void
XTypeWatch::track_branch(Window w)
{
    if (w == window_)		// don't want to mess with my own window(s)
	return ;
    {
	/*
	  fix for broken programs
	 */
	char *c_p =0;
	XFetchName(display_, w, &c_p);
	bool cont=true;
	if (c_p) {
	    cont |= strncmp(c_p, "Xdvi", 4);
	    cont |= strncmp(c_p, "Ghostview", 9);
	    cerr << "testing " << c_p << endl;
	}
	XFree (c_p);
	
	if (!cont) {

	    return;
	}
	
    }
    track_window(w);
    
    unsigned int no_ch;
    Window root_ret, par_ret, *child_ret;

    Status ok=XQueryTree(display_, w, &root_ret, &par_ret, &child_ret, &no_ch);
    if ( !ok ) {
	cerr << "Can't query window tree. \n";
	return;
    }
	    

    for (unsigned int i=0; i < no_ch; i++)
	track_branch(child_ret[i]);
    XFree(child_ret);
}

void
XTypeWatch::select_recorderEvents()
{
    Window root = DefaultRootWindow(display_);
  
    // track existing windows.
    track_branch(root);
    XFlush(display_);
}

void XTypeWatch::parseargs( int argc, char *argv[] ) {
    usedlabels_ = legend_ = doterm_ = ints_ = 0;

    for ( int i = 0 ; i < 4 ; i++ )
	serial_[i] = 0;

    getResources();
    parseArgs( argc, argv );
    setFont();

  
    // use labels
    if ( !strcmp( getResource( "labels" ), "True" ) )
	legend_ = 1;

    // use "free" labels
    if ( !strcmp( getResource( "usedlabels" ), "True" ) )
	usedlabels_ = 1;

    const char *bs=getResource( "microBreak" );
    micro_.parseMicro(bs);
    bs=getResource( "break" );  
    normal_.parseMacro(bs);      
}
const char *XTypeWatch::winname( void ){
    static char name[100];
    char host[100];
    strcpy( name, NAME );
    gethostname( host, 99 );
    strcat( name, host );
    return name;
}

void
XTypeWatch::resize( void ){
    int newwidth = width_ - xoff_ - 5;
    int newheight = (height_ - (10 + 5 * (nummeters_ - 1) + 
				nummeters_ * yoff_)) / nummeters_;

    int counter = 1;
    MeterNode *tmp = meters_;
    while ( tmp != NULL ) {
	tmp->meter_->resize( xoff_, 5 * counter + counter * yoff_ +
			     (counter - 1) * newheight, newwidth, newheight );
	tmp = tmp->next_;
	counter++;
    }
}

XTypeWatch::~XTypeWatch( void )
{    
    MeterNode *tmp = meters_;
    while ( tmp != NULL ){
	MeterNode *save = tmp->next_;
	delete tmp->meter_;
	delete tmp;
	tmp = save;
    }
}

void XTypeWatch::draw( void ){
    clear();
    MeterNode *tmp = meters_;
    while ( tmp != NULL ){
	tmp->meter_->draw();
	tmp = tmp->next_;
    }
}


void
XTypeWatch::run( void ){

    while( !done_ ){

	checkevent();
	
	MeterNode *tmp = meters_;
	while ( tmp != NULL ){
	    if ( tmp->meter_->requestevent() )
		tmp->meter_->checkevent();
	    tmp = tmp->next_;
	}

	flush();
	usleep( WAITTIME * 1000 );
	updateRecorder();
    }
}


void
XTypeWatch::updateRecorder()
{
    recorder_->update(myTime_);
}
void
XTypeWatch::usleep( unsigned long usec )
{
    myTime_ += usec / 1000;
    struct timeval time;

    time.tv_sec = usec / 1000000;
    time.tv_usec = usec - time.tv_sec * 1000000;

    select( 0, 0, 0, 0, &time );
}


void
XTypeWatch::keyPressEvent( XKeyEvent &event )
{
    recorder_->record(event, myTime_);
}


void
XTypeWatch::winCreateEvent(XCreateWindowEvent &event)
{
    track_branch(event.window);
}

